@@ -259,7 +259,9 @@ tb_process_ref_t tb_process_init_cmd(tb_char_t const* cmd, tb_process_attr_ref_t
         tb_bool_t detach = attr && (attr->flags & TB_PROCESS_FLAG_DETACH);
 
         // init flags
+        // see: https://learn.microsoft.com/en-us/windows/win32/procthread/process-creation-flags
         DWORD flags = CREATE_UNICODE_ENVIRONMENT;
+        if (attr && attr->flags & TB_PROCESS_FLAG_NO_WINDOW) flags |= CREATE_NO_WINDOW;
         if (attr && attr->flags & TB_PROCESS_FLAG_SUSPEND) flags |= CREATE_SUSPENDED;
         if (!detach) flags |= CREATE_BREAKAWAY_FROM_JOB; // create process with parent process group by default, need set JOB_OBJECT_LIMIT_BREAKAWAY_OK limit for job
 
@@ -434,6 +436,17 @@ tb_process_ref_t tb_process_init_cmd(tb_char_t const* cmd, tb_process_attr_ref_t
                 // enable handles
                 HANDLE hStdError = attr->errtype == TB_PROCESS_REDIRECT_TYPE_PIPE? tb_pipe_file_handle(attr->err.pipe) : (HANDLE)attr->err.file;
 
+                // we need duplicate it if output to stdout/stderr pipes in same time
+                if (attr->outtype == TB_PROCESS_REDIRECT_TYPE_PIPE && attr->out.pipe &&
+                    attr->errtype == TB_PROCESS_REDIRECT_TYPE_PIPE && attr->err.pipe &&
+                    attr->out.pipe == attr->err.pipe)
+                {
+                    HANDLE hStdOutput = hStdError;
+                    if (!DuplicateHandle(GetCurrentProcess(), hStdOutput, GetCurrentProcess(), &hStdError, 0, TRUE, DUPLICATE_SAME_ACCESS))
+                        break;
+                    process->file_handles[process->file_handles_count++] = hStdError;
+                }
+
                 // enable inherit
                 tb_kernel32()->SetHandleInformation(hStdError, HANDLE_FLAG_INHERIT, TRUE);
                 handlesToInherit[handlesToInheritCount++] = hStdError;
