@@ -2,10 +2,10 @@
 set_project("tbox")
 
 -- set xmake minimum version
-set_xmakever("2.6.1")
+set_xmakever("2.8.2")
 
 -- set project version
-set_version("1.7.4", {build = "%Y%m%d", soname = true})
+set_version("1.7.5", {build = "%Y%m%d", soname = true})
 
 -- set warning all as error
 set_warnings("all", "error")
@@ -18,14 +18,23 @@ set_languages(stdc)
 set_configvar("_GNU_SOURCE", 1)
 set_configvar("_REENTRANT", 1)
 
+-- add module directories
+add_moduledirs("xmake")
+
 -- disable some compiler errors
-add_cxflags("-Wno-error=deprecated-declarations", "-fno-strict-aliasing", "-Wno-error=expansion-to-defined")
-add_mxflags("-Wno-error=deprecated-declarations", "-fno-strict-aliasing", "-Wno-error=expansion-to-defined")
+add_cxflags("-Wno-error=deprecated-declarations", "-fno-strict-aliasing", "-Wno-error=expansion-to-defined", "-Wno-error=empty-body")
+add_mxflags("-Wno-error=deprecated-declarations", "-fno-strict-aliasing", "-Wno-error=expansion-to-defined", "-Wno-error=empty-body")
 if has_config("coroutine") then
     -- https://github.com/tboox/tbox/issues/218
     add_cxflags("gcc::-Wno-error=dangling-pointer")
 end
 
+-- set wasm toolchain
+if is_plat("wasm") then
+    add_requires("emscripten")
+    set_toolchains("emcc@emscripten")
+end
+
 -- add build modes
 add_rules("mode.release", "mode.debug", "mode.profile", "mode.coverage", "mode.valgrind", "mode.asan", "mode.tsan", "mode.ubsan")
 if is_mode("debug") then
@@ -64,11 +73,11 @@ if is_plat("windows") then
     else
         set_runtimes("MT")
     end
-    add_syslinks("ws2_32")
+    add_syslinks("ws2_32", "user32")
 elseif is_plat("android") then
     add_syslinks("m", "c")
 elseif is_plat("mingw", "msys", "cygwin") then
-    add_syslinks("ws2_32", "pthread", "m")
+    add_syslinks("ws2_32", "user32", "pthread", "m")
 elseif is_plat("haiku") then
     add_syslinks("pthread", "network", "m", "c")
 else
